config = {'schemas': 'bin.sat.schemas', 'commands': {'maple': {'maple_stock': 'tools/sat/maple/MapleLRB_LCMoccRestart/sources/simp/glucose_static -cpu-lim=33'}, 'minisat': {'minisat_clone': 'minisat -cpu-lim=33', 'minisat_stock': 'minisat -cpu-lim=33'}, 'syrup': {'syrup_stock': 'tools/sat/syrup/syrup/bin/glucose-syrup -nthreads=8 -model -maxmemory=50000 -cpu-lim=33'}}, 'handlers': {'maple': 'bin.sat.maple_handler', 'minisat': 'bin.sat.minisat_handler', 'syrup': 'bin.sat.syrup_handler'}, 'timeout': 30.0, 'instances': 'bin/sat/instances', 'database_name': 'sat_database'}
